/* global BDF,Ficheform,$$,ID */

Ficheform.Init = {};

Ficheform.Init.datation = function () {
    BDF.$({ficheformEntry: "datation-unique"}).each(function (index, element) {
        let $entry = $(element);
        let input = BDF.$($entry, {ficheformRole: "textinput"})[0];
        let genId = BDF.replaceWithDateInputs(input);
        $entry.find(".ficheform-Small").removeClass("ficheform-Small").addClass("ficheform-Large");
        $entry.find(".ficheform-Medium").removeClass("ficheform-Medium").addClass("ficheform-Large");
        BDF.$($entry, {ficheformRole: "label"}).attr("for", genId + "_year");
    });
    
};

Ficheform.Init.geocodage = function () {
    BDF.$({ficheformEntry: "geocodage"}).each(function (index, element) {
        let $entry = $(element);
        let buttonId = BDF.generateId();
        let latitudeId = BDF.$($entry, {ficheformRole: "latitude"})[0].id;
        let longitudeId = BDF.$($entry, {ficheformRole: "longitude"})[0].id;
        $entry.find("tr:first").append(BDF.render("ficheform:geocodage", {
            buttonId: buttonId,
            noTab: Ficheform.getOption("noTab")
        }));
        $$(buttonId).click(function () {
            _piocheGeocodage(latitudeId, longitudeId, $entry.data("ficheformParam"));
        });
    });
    
    
    function _piocheGeocodage(latitudeId, longitudeId, param) {
        let lat = ID(latitudeId).value;
        let lon = ID(longitudeId).value;
        let appelant = latitudeId + "," + longitudeId;
        let url = "pioche?page=geocodage&appelant="+ appelant + "&lat=" + encodeURIComponent(lat) + "&lon=" + encodeURIComponent(lon);
        if (param) {
            url = url + "&param=" + param;
        }
        Ficheform.openDialog(url,"pioche_geocodage",620,660);
    }
};

Ficheform.Init.grouped = function () {
    BDF.$({_element: "input", ficheformGrouped: "1"}).each(function (index, element) {
        if (element.checked) {
            $(element).parent().addClass("ficheform-SelectedLabel");
        }
        $(element).change(_testChange);
    });
    
    
    function _testChange() {
        let type = this.type;
        if (type === "radio") {
            let name = this.name;
            BDF.$({_element: "input", _name: name}).parent().removeClass("ficheform-SelectedLabel");
        }
        $(this).parent().toggleClass("ficheform-SelectedLabel", this.checked);
    }
};

Ficheform.Init.mandatory = function () {
    var $mandatoryList = BDF.$({ficheformMandatory: "1"});
    BDF.$($mandatoryList, {ficheformRole: "label"}).append(BDF.render("ficheform:mandatoryicon"), {});
};

Ficheform.Init.piocheitem = function () {
    _initByPage("langue","lang-code","action-ficheform-Lang","_ link.pioche.langue");
    _initByPage("pays","country-code","action-ficheform-Country","_ link.pioche.pays");
    _initByPage("redacteur","redacteur-code","action-ficheform-Redacteur","_ link.pioche.redacteur");


    function _initByPage(page, entryType, action, locKey) {
         BDF.$({ficheformEntry: entryType}).each(function (index, element) {
            let $entry = $(element);
            let entryKey = $entry.data("ficheformKey");
            let buttonId = BDF.generateId();
            let $input = BDF.$($entry, {ficheformRole: "textinput"});
            let inputId = $input.attr("id");
            $input.after(BDF.render("ficheform:pioche", {
                buttonId: buttonId,
                action: action,
                locKey: locKey,
                noTab: Ficheform.getOption("noTab")
            }));
            $$(buttonId).click(function () {
                _piocheItem(page, entryKey, inputId);
            });
        });
    }
    
    function _piocheItem(page, entryKey, inputId) {
        let $entry = Ficheform.$entry(entryKey);
        let limit = $entry.data("ficheformLimit");
        if (!limit) {
            limit = -1;
        }
        let url = "pioche?page=" + page + "&appelant=" + inputId + "&limit=" + limit + "&subsets=";
        let param = $entry.data("ficheformParam");
        if (param) {
            url = url + "&param=" + param;
        }
        Ficheform.openDialog(url,"pioche_" + entryKey,430,510);
    }
};

Ficheform.Init.maxlength = function () {
    BDF.$({ficheformEntry: "text-maxlength"}).each(function (index, element) {
        let $input = BDF.$($(element), {ficheformRole: "textinput"});
        let maxLength = parseInt($input.attr("maxlength"));
        if (maxLength > 0) {
            let inputlengthId = BDF.generateId();
            let startLength = $input.val().length;
            $input.after(BDF.render("ficheform:maxlength", {
               inputlengthId: inputlengthId,
               maxLength: maxLength,
               startLength: startLength
            }));
            _checkClasses($input, $$(inputlengthId), startLength, maxLength);
            $input.on("input", function () {
                let $this = $(this);
                let newLength = $this.val().length;
                $$(inputlengthId, "length").text(newLength);
                _checkClasses($this, $$(inputlengthId), newLength, maxLength);
            });
        }
    });
    
    
    function _checkClasses($input, $inputLength, currentLength, maxLength) {
        if (currentLength > maxLength) {
            $input.removeClass("ficheform-MaxReached").addClass("ficheform-MaxExceeded");
            $inputLength.removeClass("ficheform-MaxReached").addClass("ficheform-MaxExceeded");
        } else if (currentLength === maxLength) {
            $input.addClass("ficheform-MaxReached").removeClass("ficheform-MaxExceeded");
            $inputLength.addClass("ficheform-MaxReached").removeClass("ficheform-MaxExceeded");
        } else {
            $input.removeClass("ficheform-MaxReached").removeClass("ficheform-MaxExceeded");
            $inputLength.removeClass("ficheform-MaxReached").removeClass("ficheform-MaxExceeded");
        }
    }
    
};


$(function () {
    BDF.initTemplates();
    Ficheform.Init.datation();
    Ficheform.Init.geocodage();
    Ficheform.Init.grouped();
    Ficheform.Init.mandatory();
    Ficheform.Init.piocheitem();
    Ficheform.Init.maxlength();
    Ficheform.ThesaurusInclude.init();
    Ficheform.CorpusInclude.init();
    Ficheform.Section.init();
    Ficheform.AlbumInclude.init();
    Ficheform.AddendaInclude.init();
    Ficheform.Suggestion.init();
    Ficheform.init();
});